@@ -25,6 +25,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <fcntl.h>
 
 #include "ext/fmemopen.h" /* For OS not yet implementing this new standard function */
 #include "ffutil.h"
@@ -48,12 +49,64 @@ ffindex_db_t * ffindex_index_db_open(ffindex_db_t * ffindex_db)
     return NULL;
 
   ffindex_db->ffindex = ffindex_index_parse(ffindex_db->ffindex_file, ffindex_db->num_max_entries);
+
+  if(!ffindex_db->ffindex)
+  {
+    fferror_print(__FILE__, __LINE__, __func__, ffindex_db->ffindex_filename);
+    return NULL;
+  }
+
   ffindex_db->ffdata = ffindex_mmap_data(ffindex_db->ffdata_file, &ffindex_db->ffdata_size);
 
+  if(ffindex_db->ffdata_size == 0)
+    warn("Problem mapping index file. Is it empty or is another process reading it?");
+
+  if(ffindex_db->ffdata == MAP_FAILED)
+  {
+    fferror_print(__FILE__, __LINE__, __func__, ffindex_db->ffdata_filename);
+    return NULL;
+  }
+  ffindex_db->ffindex->type = SORTED_ARRAY; /* XXX Assume a sorted file for now */
+
   return ffindex_db;
 }
 
 
+int ffindex_index_db_close(ffindex_db_t* ffindex_db)
+{
+  int ret = EXIT_SUCCESS;
+
+  ffindex_index_close(ffindex_db->ffindex);
+
+  ret = munmap(ffindex_db->ffdata, ffindex_db->ffdata_size);
+  if(ret)
+  {
+    fferror_print(__FILE__, __LINE__, __func__, "munmap");
+    return ret;
+  }
+
+  free(ffindex_db);
+
+  return EXIT_SUCCESS;
+}
+
+
+int ffindex_index_close(ffindex_index_t* ffindex)
+{
+  size_t nbytes = sizeof(ffindex_index_t) + (sizeof(ffindex_entry_t) * ffindex->num_max_entries);
+
+  int ret = munmap(ffindex, nbytes);
+
+  if(ret)
+  {
+    fferror_print(__FILE__, __LINE__, __func__, "munmap");
+    return ret;
+  }
+
+  return EXIT_SUCCESS;
+}
+
+
 /* return *out_data_file, *out_index_file, out_offset.
  Setting to a given offset could be supported with a special mode.
  */
@@ -139,13 +192,13 @@ EXCEPTION_ffindex_insert_memory:
 }
 
 
-/* Insert all file from a list into ffindex */
+/* Insert all files from a list into ffindex */
 int ffindex_insert_list_file(FILE *data_file, FILE *index_file, size_t *start_offset, FILE *list_file)
 {
   size_t offset = *start_offset;
   char path[PATH_MAX];
   while(fgets(path, PATH_MAX, list_file) != NULL)
-    ffindex_insert_file(data_file, index_file, &offset, ffnchomp(path, strlen(path)), basename(path));
+    ffindex_insert_file(data_file, index_file, &offset, ffnchomp(path, strnlen(path, PATH_MAX)), basename(path));
 
   /* update return value */
   *start_offset = offset;
@@ -163,7 +216,7 @@ int ffindex_insert_dir(FILE *data_file, FILE *index_file, size_t *start_offset,
     return -1;
   }
 
-  size_t input_dir_name_len = strlen(input_dir_name);
+  size_t input_dir_name_len = strnlen(input_dir_name, FILENAME_MAX);
   char path[PATH_MAX];
   strncpy(path, input_dir_name, NAME_MAX);
   if(input_dir_name[input_dir_name_len - 1] != '/')
@@ -291,6 +344,19 @@ int ffindex_compare_entries_by_name(const void *pentry1, const void *pentry2)
   return strncmp(entry1->name, entry2->name, FFINDEX_MAX_ENTRY_NAME_LENTH);
 }
 
+
+ffindex_entry_t* ffindex_grep_entry_by_str(ffindex_index_t *ffindex, char *name, size_t* offset)
+{
+  for(size_t i = *offset; i < ffindex->n_entries; i++)
+    if(strstr(ffindex->entries[i].name, name))
+    {
+      *offset = i;
+      return &ffindex->entries[i];
+    }
+  return NULL;
+}
+
+
 ffindex_entry_t* ffindex_get_entry_by_name(ffindex_index_t *index, char *name)
 {
   return ffindex_bsearch_get_entry(index, name);
@@ -308,11 +374,13 @@ ffindex_index_t* ffindex_index_parse(FILE *index_file, size_t num_max_entries)
 {
   if(num_max_entries == 0)
     num_max_entries = FFINDEX_MAX_INDEX_ENTRIES_DEFAULT;
+
   size_t nbytes = sizeof(ffindex_index_t) + (sizeof(ffindex_entry_t) * num_max_entries);
-  ffindex_index_t *index = (ffindex_index_t *)malloc(nbytes);
-  if(index == NULL)
+  ffindex_index_t *index = (ffindex_index_t *)mmap(0, nbytes, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS, -1, 0);
+
+  if(index == MAP_FAILED)
   {
-    fprintf(stderr, "Failed to allocate %ld bytes\n", nbytes);
+    fprintf(stderr, "Failed to allocate %ld bytes for index\n", nbytes);
     fferror_print(__FILE__, __LINE__, __func__, "malloc failed");
     return NULL;
   }
@@ -320,21 +388,33 @@ ffindex_index_t* ffindex_index_parse(FILE *index_file, size_t num_max_entries)
 
   index->file = index_file;
   index->index_data = ffindex_mmap_data(index_file, &(index->index_data_size));
+
   if(index->index_data_size == 0)
-    warn("Problem with data file. Is it empty or is another process reading it?");
+    warn("Problem mapping index file. Is it empty or is another process reading it?");
+
   if(index->index_data == MAP_FAILED)
   {
-    free(index);
+    ffindex_index_close(index);
     return NULL;
   }
   index->type = SORTED_ARRAY; /* XXX Assume a sorted file for now */
 
+  posix_madvise(index->index_data, index->index_data_size, POSIX_MADV_SEQUENTIAL);
+
   /* Faster than scanf per line */
   size_t names_too_long = 0;
   const char* d = index->index_data;
-  int i;
+  size_t i;
   for(i = 0; d < (index->index_data + index->index_data_size); i++)
   {
+    if(i >= index->num_max_entries)
+    {
+      errno = EINVAL;
+      fferror_print(__FILE__, __LINE__, __func__, "too many entries in index");
+      munmap(index->index_data, index->index_data_size);
+      ffindex_index_close(index);
+      return NULL;
+    }
     const char* end;
     int p;
     for(p = 0; *d != '\t'; d++)
@@ -343,14 +423,36 @@ ffindex_index_t* ffindex_index_parse(FILE *index_file, size_t num_max_entries)
       else
         names_too_long++;
     index->entries[i].name[p] = '\0';
-    index->entries[i].offset = parse_ulong(++d, &end);
+    index->entries[i].offset = ffparse_ulong(++d, &end);
     d = end;
-    index->entries[i].length  = parse_ulong(++d, &end);
+    index->entries[i].length = ffparse_ulong(++d, &end);
     d = end + 1; /* +1 for newline */
   }
 
   index->n_entries = i;
 
+  int ret = 0;
+  
+  ret = munmap(index->index_data, index->index_data_size);
+
+  if(ret)
+  {
+    fferror_print(__FILE__, __LINE__, __func__, "munmap");
+    return NULL;
+  }
+
+
+  /* Make sure that the actually filled part part of the index stays
+   * in memory even under memory pressure.
+   */
+  ret = mlock(index, sizeof(ffindex_index_t) + (sizeof(ffindex_entry_t) * index->n_entries));
+
+  if(ret)
+  {
+    fferror_print(__FILE__, __LINE__, __func__, "mlock");
+    return NULL;
+  }
+
   if(index->n_entries == 0)
     warnx("index with 0 entries");
 
@@ -434,48 +536,48 @@ void ffindex_sort_index_file(ffindex_index_t *index)
 }
 
 
-int ffindex_write(ffindex_index_t* index, FILE* index_file)
+/* Faster than fprintf but no error checking.
+ * If more robustness is neeeded, prealocate the output file.
+ */
+size_t ffindex_print_entry(FILE* file, ffindex_entry_t* entry)
 {
-  /* Use tree if available */
-  if(index->type == TREE)
-    return ffindex_tree_write(index, index_file);
+  size_t written;
+  written = fwrite(entry->name, 1, strnlen(entry->name, FFINDEX_MAX_ENTRY_NAME_LENTH), file);
+  fputc('\t', file);
 
-  for(size_t i = 0; i < index->n_entries; i++)
-  {
-    ffindex_entry_t ffindex_entry = index->entries[i];
-    if(fprintf(index_file, "%s\t%zd\t%zd\n", ffindex_entry.name, ffindex_entry.offset, ffindex_entry.length) < 0)
-      return EXIT_FAILURE;
-  }
-  return EXIT_SUCCESS;
+  written += fffprint_ulong(file, entry->offset);
+  fputc('\t', file);
+
+  written += fffprint_ulong(file, entry->length);
+
+  fputc('\n', file);
+
+  return written + 3; // * fputc
 }
 
 
-ffindex_index_t* ffindex_unlink_entries(ffindex_index_t* index, char** sorted_names_to_unlink, int n_names)
+int ffindex_write(ffindex_index_t* index, FILE* index_file)
 {
-  int i = index->n_entries - 1;
-  /* walk list of names to delete */
-  for(int n = n_names - 1; n >= 0;  n--)
-  {
-    char* name_to_unlink = sorted_names_to_unlink[n];
-    /* walk index entries */
-    for(; i >= 0; i--)
-    {
-      int cmp = strncmp(name_to_unlink, index->entries[i].name, FFINDEX_MAX_ENTRY_NAME_LENTH);
-      if(cmp == 0) /* found entry */
-      {
-        /* Move entries after the unlinked ones to close the gap */
-        size_t n_entries_to_move = index->n_entries - i - 1;
-        if(n_entries_to_move > 0) /* not last element of array */
-          memmove(index->entries + i, index->entries + i + 1, n_entries_to_move * sizeof(ffindex_entry_t));
-        index->n_entries--;
-        break;
-      }
-      else if(cmp > 0) /* not found */
-        break;
-    }
-  }
+  int ret = EXIT_SUCCESS;
+#if _POSIX_C_SOURCE >= 200112L
+  ret = posix_fallocate(fileno(index_file), 0, index->n_entries * 30); // guesstimate
+  if(ret)
+    return ret;
+#endif
 
-  return index;
+  /* Use tree if available */
+  if(index->type == TREE)
+    ret = ffindex_tree_write(index, index_file);
+  else
+    for(size_t i = 0; i < index->n_entries; i++)
+      if(ffindex_print_entry(index_file,index->entries + i) < 6)
+        return EXIT_FAILURE;
+
+#if _POSIX_C_SOURCE >= 200112L
+  ftruncate(fileno(index_file), ftell(index_file));
+#endif
+
+  return ret;
 }
 
 
@@ -484,19 +586,8 @@ ffindex_index_t* ffindex_unlink(ffindex_index_t* index, char* name_to_unlink)
   /* Use tree if available */
   if(index->type == TREE)
     return ffindex_tree_unlink(index, name_to_unlink);
-
-  ffindex_entry_t* entry = ffindex_bsearch_get_entry(index, name_to_unlink);
-  if(entry == NULL)
-  {
-    fprintf(stderr, "Warning: could not find '%s'\n", name_to_unlink);
-    return index;
-  }
-  /* Move entries after the unlinked one to close the gap */
-  size_t n_entries_to_move = index->entries + index->n_entries - entry - 1;
-  if(n_entries_to_move > 0) /* not last element of array */
-    memmove(entry, entry + 1, n_entries_to_move * sizeof(ffindex_entry_t));
-  index->n_entries--;
-  return index;
+  else
+    return NULL;
 }
 
 
