@@ -21,7 +21,7 @@
 
 #include <stdio.h>
 
-#define FFINDEX_VERSION 0.996
+#define FFINDEX_VERSION 0.997
 #define FFINDEX_COPYRIGHT "\nDesigned and implemented by Andreas Hauser <andy@splashground.de>.\n"
 
 #define FFINDEX_MAX_INDEX_ENTRIES_DEFAULT 80000000
@@ -68,6 +68,11 @@ typedef struct ffindex_db
 
 ffindex_db_t * ffindex_index_db_open(ffindex_db_t * ffindex_db);
 
+int ffindex_index_db_close(ffindex_db_t* ffindex_db);
+
+
+int ffindex_index_close(ffindex_index_t* ffindex);
+
 /* return *out_data_file, *out_index_file, out_offset. */
 int ffindex_index_open(char *data_filename, char *index_filename, char* mode, FILE **out_data_file, FILE **out_index_file, size_t *out_offset);
 
@@ -103,6 +108,9 @@ char* ffindex_get_data_by_name(char *data, ffindex_index_t *index, char *name);
 char* ffindex_get_data_by_index(char *data, ffindex_index_t *index, size_t entry_index);
 
 
+ffindex_entry_t* ffindex_grep_entry_by_str(ffindex_index_t *ffindex, char *name, size_t* offset);
+
+
 ffindex_entry_t* ffindex_get_entry_by_index(ffindex_index_t *index, size_t entry_index);
 
 ffindex_entry_t* ffindex_get_entry_by_name(ffindex_index_t *index, char *name);
@@ -114,6 +122,8 @@ ffindex_entry_t* ffindex_bsearch_get_entry(ffindex_index_t *index, char *name);
 
 void ffindex_sort_index_file(ffindex_index_t *index);
 
+size_t ffindex_print_entry(FILE* file, ffindex_entry_t* entry);
+
 int ffindex_write(ffindex_index_t* index, FILE* index_file);
 
 ffindex_index_t* ffindex_unlink(ffindex_index_t* index, char *entry_name);
