@@ -25,9 +25,11 @@
 #include <sys/wait.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <signal.h>
 
-
-#include <mpi.h>
+#ifdef HAVE_MPI
+# include <mpi.h>
+#endif
 
 
 #include "ffindex.h"
@@ -161,13 +163,18 @@ int ffindex_apply_by_entry(char *data, ffindex_index_t* index, ffindex_entry_t*
 
 int main(int argn, char **argv)
 {
-  int mpi_error,
-      mpi_rank,
+  int mpi_rank,
       mpi_num_procs;
 
+#ifdef HAVE_MPI
+  int mpi_error;
   mpi_error = MPI_Init(&argn, &argv);                        if(mpi_error) goto EXCEPTION;
   mpi_error = MPI_Comm_rank(MPI_COMM_WORLD, &mpi_rank);      if(mpi_error) goto EXCEPTION;
   mpi_error = MPI_Comm_size(MPI_COMM_WORLD, &mpi_num_procs); if(mpi_error) goto EXCEPTION;
+#else
+  mpi_rank = 0;
+  mpi_num_procs = 1;
+#endif
 
   int opt, merge = 1;
   char *data_filename_out  = NULL,
@@ -201,12 +208,13 @@ int main(int argn, char **argv)
                     "\t-l\tLOG_FILE like an FFindex file but with the return value of apply added.\n"
                     "\t\tThis file can be processed and used as FFindex file.\n"
                     "\t\tE.g. grep \"\t1$\" foo.log | cut -f 1-3 > foo-special.ffindex.\n"
-                    "\t\twhere maybe the ffindex_apply_mpi ran with -- perl -e 'return 1 if(/^SPECIAL/)'\n"
+                    "\t\twhere maybe the %s ran with -- perl -e 'return 1 if(/^SPECIAL/)'\n"
                     "\t-m\tDo not merge the FFindex parts generated by the different MPI processes\n"
                     "\t\tThis is useful for large MPI Jobs where merge time might be accounted.\n"
                     "\t-d\tFFDATA output file, stdout of program will be captured\n"
                     "\t-i\tFFINDEX output file, entries get the same name as input entry\n"
                     FFINDEX_COPYRIGHT,
+                    argv[0],
                     argv[0]);
     errno = EINVAL;
     goto EXCEPTION;
@@ -257,7 +265,9 @@ int main(int argn, char **argv)
   if(index == NULL)
   {
     fferror_print(__FILE__, __LINE__, "ffindex_index_parse", index_filename);
+#ifdef HAVE_MPI
     MPI_Finalize();
+#endif
     exit(EXIT_FAILURE);
   }
   
@@ -306,7 +316,9 @@ int main(int argn, char **argv)
   if(index_file_out != NULL)
     fclose(index_file_out);
 
+#ifdef HAVE_MPI
   MPI_Barrier(MPI_COMM_WORLD);
+#endif
 
 
   // merge FFindexes in master
@@ -333,7 +345,9 @@ int main(int argn, char **argv)
     }
   }
 
+#ifdef HAVE_MPI
   MPI_Finalize();
+#endif
 
   return EXIT_SUCCESS;
 
@@ -343,7 +357,9 @@ EXCEPTION:
   perror(argv[0]);
   fprintf(stderr, "\nEXCEPTION ffindex_apply_mpi in rank: %d\n", mpi_rank);
 
+#ifdef HAVE_MPI
   MPI_Finalize();
+#endif
 
   return EXIT_FAILURE;
 }
