@@ -26,6 +26,7 @@ void usage(char* program_name)
 {
     fprintf(stderr, "USAGE: %s data_filename index_filename entry name(s)\n"
                     "\t-f FILE\t\tfile containing a list of file names, one per line\n"
+                    "\t-h\t\tdisplay this help message\n"
                     "\t-n\t\tuse index of entry instead of entry name\n"
                     FFINDEX_COPYRIGHT,
                     program_name);
@@ -37,7 +38,8 @@ int main(int argn, char **argv)
   int opt;
   char* list_filenames[4096];
   size_t list_filenames_index = 0;
-  while ((opt = getopt(argn, argv, "f:n")) != -1)
+  char* program_name = basename(argv[0]);
+  while ((opt = getopt(argn, argv, "f:hn")) != -1)
   {
     switch (opt)
     {
@@ -47,8 +49,11 @@ int main(int argn, char **argv)
       case 'n':
         by_index = 1;
         break;
+      case 'h':
+        usage(program_name);
+        return EXIT_SUCCESS;
       default:
-        usage(argv[0]);
+        usage(program_name);
         return EXIT_FAILURE;
     }
   }
@@ -68,7 +73,6 @@ int main(int argn, char **argv)
   ffindex_db = ffindex_index_db_open(ffindex_db);
   if(!ffindex_db)
   {
-    errno = EINVAL;
     fferror_print(__FILE__, __LINE__, "ffindex_index_db_open failed", argv[0]);
     return EXIT_FAILURE;
   }
@@ -111,7 +115,7 @@ int main(int argn, char **argv)
         char name[PATH_MAX];
         while(fgets(name, PATH_MAX, list_file) != NULL)
         {
-          ffindex_entry_t* entry = ffindex_get_entry_by_name(ffindex_db->ffindex, ffnchomp(name, strlen(name)));
+          ffindex_entry_t* entry = ffindex_get_entry_by_name(ffindex_db->ffindex, ffnchomp(name, strnlen(name, PATH_MAX)));
           if(entry == NULL)
           {
             errno = ENOENT; 
@@ -173,6 +177,8 @@ int main(int argn, char **argv)
          */
   }
 
+  ffindex_index_db_close(ffindex_db);
+
   return 0;
 }
 
