@@ -122,7 +122,7 @@ int main(int argn, char **argv)
           /* unlink entries in file, one per line */
           char path[PATH_MAX];
           while(fgets(path, PATH_MAX, list_file) != NULL)
-            index = ffindex_unlink(index, ffnchomp(path, strlen(path)));
+            index = ffindex_unlink(index, ffnchomp(path, strnlen(path, PATH_MAX)));
         }
 
       /* unlink entries specified by args */
@@ -134,17 +134,17 @@ int main(int argn, char **argv)
   }
 
   if(sort)
-  {
     ffindex_sort_index_file(index);
-    index_file = fopen(index_filename, "w");
-    if(index_file == NULL) { perror(index_filename); return EXIT_FAILURE; }
-    err += ffindex_write(index, index_file);
-  }
 
   /* Write index back */
   index_file = fopen(index_filename, "w");
   if(index_file == NULL) { perror(index_filename); return EXIT_FAILURE; }
-  err += ffindex_write(index, index_file);
+  err = ffindex_write(index, index_file);
+  if(err)
+    fferror_print(__FILE__, __LINE__, __func__, index_filename);
+
+  ffindex_index_close(index);
+
   return err;
 }
 
